//------------------------------------------------------------------------------
// Name: TUSB3410Demo.cpp
// Func: PC demo application for establishing a serial connection with the
//       MSP430-TUSB3410 Reference Design board. The program is part of the
//       "MSP430 USB Connectivity using TUSB3410" application note (SLAA276).
//       The demo scans all system COM class devices for the TUSB3410 virtual
//       COM port (VCP) with the hardware ID as configured in the Windows INF
//       driver files. If found, the VCP is opened with a baud rate of 460,800.
//       The lower nibble of a received character is output to the console.
//       Also, on press or release of the keys 1, 2, 3, and 4, a byte is
//       transmitted back to MSP430 demo board containing the updated button
//       state. The program is end by pressing <ESC>.
// Ver.: 1.00
// Date: November 2005
// Auth: Andreas Dannenberg
//       MSP430/TMS470 Applications
//       Texas Instruments Inc.
// Rem.: - File can be build as a console application with any common Win32
//         compiler such as Borland C++ or Microsoft Visual C++
//       - To successful build this project, the library file 'setupapi.lib'
//         has to be linked to the project
//------------------------------------------------------------------------------

// Standard C header files
#include <stdlib.h>
#include <stdio.h>

// Windows header files
#define WIN32_LEAN_AND_MEAN   // Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <setupapi.h>         // Link library setupapi.lib to the project
#include "defines.h"
#include <time.h>
#define _CRT_SECURE_NO_WARNINGS
#define NL	printf("\n")
#define MYBYTESIZE 256
//------------------------------------------------------------------------------
// Misc definitions
//------------------------------------------------------------------------------
#define ERROR_WRONG_COM       0xe0000000    // Custom error code, bit 29 is set
#define MAX_COM_SEARCH_INDEX  100           // Max COM # to check
#define NICHTNOETIG
//------------------------------------------------------------------------------
// Globals
//------------------------------------------------------------------------------
HANDLE hCom;
OVERLAPPED OverlappedIn = { 0, 0, 0, 0, 0 };
OVERLAPPED OverlappedOut = { 0, 0, 0, 0, 0 };
BYTE COMBuffer[100];
//------------------------------------------------------------------------------
// Define a GUID as a constant located in this module, instead of as an
// external reference to the constant located in another module.
// (extract from guiddef.h)
//------------------------------------------------------------------------------
#define DECLSPEC_SELECTANY  __declspec(selectany)
#undef DEFINE_GUID
#define DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
          EXTERN_C const GUID DECLSPEC_SELECTANY name \
          = { l, w1, w2, { b1, b2,  b3,  b4,  b5,  b6,  b7,  b8 } }

// CDC prototypes
BOOL PacketMenu(BYTE *Array,DWORD *Arraylength);
void printbyte(BYTE *bytes, ULONG bytenum);
void printbinary(BYTE myByte,int iBits);
BOOL Splitbyte(BYTE byte, int bit);
void wait(int sec);
// CDC prototypes end
//------------------------------------------------------------------------------
// Define the GUID for the COM PORT CLASS of interface. This GUID is used
// by EnumComPorts() to enumerate all existing COM port devices.
//------------------------------------------------------------------------------
DEFINE_GUID(GUID_CLASS_COMPORT, 0x86e0d1e0L, 0x8089, 0x11d0,
            0x9c, 0xe4, 0x08, 0x00, 0x3e, 0x30, 0x1f, 0x73);
//------------------------------------------------------------------------------
// DWORD WINAPI EnumComPorts(DWORD dwIndex, LPTSTR lpszName)
//
// User-mode code fragment to identify which COMnn port[s] are attached
// to the Texas Instrument UMP (the TUSB3410 or the TUSB5052).
// It checks the hardware ID of the virtual COM port (hard-coded in the
// TI driver INF files) and returns the COMnn port which the OS embeds in
// the "Friendly Name". When called with dwIndex = 0, the function will
// enumerate all COMPORT class devices and return the name of the first
// one that was found. Subsequent calls using an incremented dwIndex
// parameter can be performed until ERROR_NO_MORE_ITEMS is returned.
//
// The TUSB3410 driver includes the ability to customize the "Friendly Name"
// before the "(COMnn)", and the HWID after the "umpport\". The search string
// can be modified here.
// further.
//
// IN:  dwIndex               CCOMPORT class device # 0, 1, 2, ... to check
// OUT: lpszName              COMnn name of given device #
//      return()              ERROR_SUCCESS - lpszName is valid
//                            ERROR_WRONG_COM - A COM port was found, but is
//                                              not the one we are looking for
//                            ERROR_NO_MORE_ITEMS - End of device list reached.
//------------------------------------------------------------------------------
DWORD WINAPI EnumComPorts(DWORD dwIndex, LPTSTR lpszName)
{
  //const TCHAR szHardwareIDUmp[] = "umpport\\VID_0451_BEEF_com";
  const TCHAR szHardwareIDUmp[] = "umpport\\CDCEL9xx_FAMILY_EVM_COM";
  static HDEVINFO g_hdi = INVALID_HANDLE_VALUE; // Setup API enum handle
  SP_DEVICE_INTERFACE_DATA did;
  SP_DEVINFO_DATA devinfo;
  PSP_DEVICE_INTERFACE_DETAIL_DATA pidd;
  DWORD dwNeeded = 0;
  TCHAR szHardwareID[60];
  TCHAR szFriendlyName[60];
  PTCHAR pcParse;

  lpszName[0] = 0;                              // Default to empty string

  // Open new enumeration if needed
  if (dwIndex == 0)
  {
    // Close open device interface enum set if any
    if (g_hdi != INVALID_HANDLE_VALUE)
      SetupDiDestroyDeviceInfoList(g_hdi);

    // Enumerate all devices present (DIGCF_PRESENT)
    g_hdi = (HDEVINFO)SetupDiGetClassDevs((CONST LPGUID)&GUID_CLASS_COMPORT,
      NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);

    if (g_hdi == INVALID_HANDLE_VALUE)
    {
      return GetLastError();
    }
  }

  // Get next device interface data ('dwIndex')
  did.cbSize = sizeof(did);

  if (!SetupDiEnumDeviceInterfaces(g_hdi, NULL,
      (CONST LPGUID)&GUID_CLASS_COMPORT, dwIndex, &did))
  {
    return GetLastError();
  }

  // Get interface detail data size
  // (Note: function returns ERROR_INSUFFICIENT_BUFFER, and sets the
  // parameter dwNeeded accordingly)
  if (!SetupDiGetDeviceInterfaceDetail(g_hdi, &did, NULL, 0, &dwNeeded, NULL))
  {
    // Ignore ERROR_INSUFFICIENT_BUFFER
    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
      return GetLastError();
  }

  // Allocate the buffer for interface details
  pidd = (PSP_DEVICE_INTERFACE_DETAIL_DATA)calloc(dwNeeded, 1);

  if (pidd == NULL)
  {
    return ERROR_NOT_ENOUGH_MEMORY;
  }

  pidd->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
  devinfo.cbSize = sizeof(devinfo);

  // Get interface details
  if (!SetupDiGetDeviceInterfaceDetail(g_hdi,
        &did, pidd, dwNeeded, NULL, &devinfo))
  {
    return GetLastError();
  }

  // Free allocated memory
  free(pidd);

  // Read out hardware ID from registry
  if (!SetupDiGetDeviceRegistryProperty(
    g_hdi,
    &devinfo,
    SPDRP_HARDWAREID,
    NULL,
    (unsigned char *)szHardwareID,
    sizeof(szHardwareID) - 1,
    NULL))
  {
    // Ignore ERROR_INSUFFICIENT_BUFFER
    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
      return GetLastError();
  }

  // Compare hardware ID, exit if mismatch
  if (strstr(szHardwareID, szHardwareIDUmp) == NULL)
  {
    return ERROR_WRONG_COM;
  }

  // Read out friendly name from registry
  if (!SetupDiGetDeviceRegistryProperty(
      g_hdi,
      &devinfo,
      SPDRP_FRIENDLYNAME,
      NULL,
      (unsigned char *)szFriendlyName,
      sizeof(szFriendlyName) - 1,
      NULL))
  {
    // Ignore ERROR_INSUFFICIENT_BUFFER
    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
      return GetLastError();
  }

  // Find pointer to "COM" substring
  pcParse = strstr(szFriendlyName, "COM");

  if (pcParse == NULL)
  {
    return ERROR_WRONG_COM;                     // "COM" not found!
  }

  // Zero-terminate COM port string after last digit
  if (!isdigit(pcParse[4]))
    pcParse[4] = 0;
  else if (!isdigit(pcParse[5]))
    pcParse[5] = 0;
  else
    pcParse[6] = 0;

  // Pass string to the return parameter
  strcpy(lpszName, pcParse);

  return ERROR_SUCCESS;
}
//------------------------------------------------------------------------------
// void CALLBACK FileIOCompletionRoutine(DWORD dwErrorCode,
//   DWORD dwNumberOfBytesTransfered, LPOVERLAPPED lpOverlapped)
//
// This function gets called upon file I/O completion. It reads out the
// received data from COMBuffer[], sets the global MSP430ButtonState variable,
// and displays the updated button status.
//
// IN:  dwErrorCode
//      dwNumberOfBytesTransfered
//      lpOverlapped
//      hCom
//      COMBuffer[]
// OUT: MSP430ButtonState
//------------------------------------------------------------------------------
void CALLBACK FileIOCompletionRoutine(DWORD dwErrorCode,
  DWORD dwNumberOfBytesTransfered, LPOVERLAPPED lpOverlapped)
{
  // Data received?
  if (dwErrorCode == 0 && dwNumberOfBytesTransfered)
  {
    // Process lower 4 bits of RX'ed byte
    if (COMBuffer[0] & 0x01)
      printf("SW1: DOWN  ");
    else
      printf("SW1: UP    ");

    if (COMBuffer[0] & 0x02)
      printf("SW2: DOWN  ");
    else
      printf("SW2: UP    ");

    if (COMBuffer[0] & 0x04)
      printf("SW3: DOWN  ");
    else
      printf("SW3: UP    ");

    if (COMBuffer[0] & 0x08)
      printf("SW4: DOWN  \n");
    else
      printf("SW4: UP    \n");
  }

  // Initiate next overlapped read operation
  ReadFileEx(hCom, COMBuffer, 1, lpOverlapped, FileIOCompletionRoutine);
}
//------------------------------------------------------------------------------
// int main(int argc, char* argv[])
//
// The main application code first scans for the attached MSP430-TUSB3410
// reference design, then opens the port and enters an event handling loop.
//
// IN:  argc                  CMD line param count - not used
//      argv                  CMD line params - not used
// OUT: return()              1 if program failed (No HW found, COM error, ...)
//                            0 otherwise
//------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
  DWORD dwIndex;
  TCHAR szDeviceName[MAX_PATH];
  TCHAR ComPort[20];
  DWORD dwReturnValue;
  DCB dcb;                                      // COM device-control block
  COMMTIMEOUTS CommTimeouts;                    // COM timeout structure
  HANDLE hConsoleInput;
  DWORD NumberOfEventsRead;
  INPUT_RECORD InputBuffer;
  BYTE PCButtonState = 0;
  BOOL TxPCButtonState = FALSE;
  BOOL ExitRequested = FALSE;
  // CDC vars
  BYTE DataArray[100], DataArray2[100];
  DWORD Arraylength;
  int j=0;
  // CDC vars end
  printf("\nMSP430-TUSB3410 Reference Design Demonstration Software V1.00\n");
  printf("Andreas Dannenberg\n");
  printf("MSP430/TMS470 Applications\n");
  printf("Texas Instruments Inc.\n");
  printf("November 2005\n\n");

  printf("Scanning system for MSP430-TUSB3410 Reference Design VCP...");

  // Scan through all COM port class devices in the system and
  // check for a friendly name / HW ID match with the MSP430 TUSB3410
  // Reference Design demo board. COM devices from 0 to MAX_COM_SEARCH_INDEX
  // are scanned.
  for (dwIndex = 0; dwIndex < MAX_COM_SEARCH_INDEX; dwIndex++)
  {
    dwReturnValue = EnumComPorts(dwIndex, szDeviceName);

    if (dwReturnValue == ERROR_SUCCESS)
    {
      break;                                    // COM port found!
    }
    if (dwReturnValue == ERROR_WRONG_COM)
    {
      continue;                                 // Keep on searching...
    }
    else if (dwReturnValue == ERROR_NO_MORE_ITEMS)
    {
      printf("not found!\n");                   // End of list reached
      return 1;
    }
    else
    {
      printf("error!\n");                       // Other error
      return 1;
    }
  }

  printf("found on %s.\n", szDeviceName);

  strcpy((CHAR*)&ComPort[0], "\\\\.\\");        // Preceed name with "\\.\" to
  strcpy((CHAR*)&ComPort[4], szDeviceName);     // access higher # ports

  hCom = CreateFile(ComPort,                    // COM-Port
    GENERIC_READ | GENERIC_WRITE,               // read/write-mode
    0,                                          // Open w/ exclusive-access
    NULL,                                       // no security attributes
    OPEN_EXISTING,                              // creation distribution
    FILE_FLAG_OVERLAPPED,                       // Use overlapped I/O
    NULL);                                      // hTemplate must be NULL

  if (hCom == INVALID_HANDLE_VALUE)             // Open successful?
  {
    printf("Error opening %s!\n", szDeviceName);
    return 1;
  }

  printf("%s opened.\n", szDeviceName);

  if (!GetCommState(hCom, &dcb))                // Read current DCB
  {
    printf("Error during GetCommState()!\n");
    CloseHandle(hCom);                          // Close COM ressource
    return 1;                                   // Exit program
  }

  dcb.BaudRate        = 460800;                 // Set baud rate
  dcb.fBinary         = TRUE;
  dcb.fParity         = FALSE;
  dcb.fOutxCtsFlow    = FALSE;                  // Disable flow-control for
  dcb.fOutxDsrFlow    = FALSE;                  // working with only 3 lines
  dcb.fDsrSensitivity = FALSE;                  // (GND, RXD, TXD)
  dcb.fOutX           = FALSE;
  dcb.fInX            = FALSE;
  dcb.fNull           = FALSE;
  dcb.fRtsControl     = RTS_CONTROL_DISABLE;    // Disable flow control
  dcb.fAbortOnError   = FALSE;
  dcb.ByteSize        = 8;
  dcb.StopBits        = ONESTOPBIT;

  if (!SetCommState(hCom, &dcb))                // Set DCB
  {
    printf("Error during SetCommState()!\n");
    CloseHandle(hCom);                          // Close COM ressource
    return 1;                                   // Exit program
  }

  CommTimeouts.ReadIntervalTimeout         = 0;
  CommTimeouts.ReadTotalTimeoutMultiplier  = 1000;  // Wait 1s per RX char
  CommTimeouts.ReadTotalTimeoutConstant    = 0;
  CommTimeouts.WriteTotalTimeoutMultiplier = 0;
  CommTimeouts.WriteTotalTimeoutConstant   = 0;

  if (!SetCommTimeouts(hCom, &CommTimeouts))    // Set COM timeouts
  {
    printf("Error during SetCommTimeouts()!\n");
    CloseHandle(hCom);                          // Close COM ressource
    return 1;                                   // Exit program
  }

  hConsoleInput = GetStdHandle(STD_INPUT_HANDLE);

  // Get handle to console user interface
  if (hConsoleInput == INVALID_HANDLE_VALUE)
  {
    printf("Error during GetStdHandle()!\n");
    CloseHandle(hCom);                          // Close COM ressource
    return 1;                                   // Exit program
  }

  // Activate UART RX functionality
  if (!ReadFileEx(hCom, COMBuffer, 1, &OverlappedIn, FileIOCompletionRoutine))
  {
    printf("Error during ReadFileEx()!\n");
    CloseHandle(hCom);                          // Close COM ressource
    return 1;                                   // Exit program
  }

  printf("\nOperate the 1, 2, 3, and 4 number keys to send data.\n");
  printf("Press <ESC> to exit this program.\t 5 for EVM menu, 6 exit\n\n");

  // Enter event processing loop...
  while (!ExitRequested)
  {
    // Check and process console events
    // Also, stay in alertable wait mode for that overlapped I/O callback
    // events can be processed.
    if (WaitForSingleObjectEx(hConsoleInput, 1, TRUE) == WAIT_OBJECT_0)
    {
      ReadConsoleInput(
        hConsoleInput,                          // Handle to console input buf
        &InputBuffer,                           // Buffer address
        1,                                      // Number of records to read
        &NumberOfEventsRead);                   // Number of records read

      if (InputBuffer.EventType == KEY_EVENT)   // Was keyboard event?
      {
        switch (InputBuffer.Event.KeyEvent.uChar.AsciiChar)
        {
          case '1' :                            // Key '1' pressed?
            if (InputBuffer.Event.KeyEvent.bKeyDown)
              PCButtonState |= 0x01;
            else
              PCButtonState &= ~0x01;
            TxPCButtonState = TRUE;             // Request TX
            break;
          case '2' :                            // Key '2' pressed?
            if (InputBuffer.Event.KeyEvent.bKeyDown)
              PCButtonState |= 0x02;
            else
              PCButtonState &= ~0x02;
            TxPCButtonState = TRUE;             // Request TX
            break;
          case '3' :                            // Key '3' pressed?
            if (InputBuffer.Event.KeyEvent.bKeyDown)
              PCButtonState |= 0x04;
            else
              PCButtonState &= ~0x04;
            TxPCButtonState = TRUE;             // Request TX
            break;
          case '4' :                            // Key '4' pressed?
            if (InputBuffer.Event.KeyEvent.bKeyDown)
              PCButtonState |= 0x08;
            else
              PCButtonState &= ~0x08;
            TxPCButtonState = TRUE;             // Request TX
            break;
		  case '5' :
			  if(PacketMenu(&DataArray[1],&Arraylength))
	    		{
				DataArray[0] = (BYTE) Arraylength;
				TxPCButtonState = TRUE;
				} 
			  break;
		  case '6' :
			  printf("\nExit\n");
			  ExitRequested = TRUE;
			  break;
          case 27 :                             // ESC-Key pressed?
            ExitRequested = TRUE;               // Request program exit
            break;
        }

        if (TxPCButtonState)                    // TX requested?
        {
          //WriteFile(hCom, &PCButtonState, 1, NULL, &OverlappedOut);
		  WriteFile(hCom, &DataArray, Arraylength+1, NULL, &OverlappedOut);
		  memset(COMBuffer,0,sizeof(COMBuffer));
		  if (!ReadFileEx(hCom, COMBuffer, 20, &OverlappedIn, FileIOCompletionRoutine))
		  {
			printf("Error during ReadFileEx()!\n");
			CloseHandle(hCom);                          // Close COM ressource
			return 1;                                   // Exit program
		  }
		  wait(1);
		  printbyte(COMBuffer,20);
          TxPCButtonState = FALSE;
        }
      }
    }
  }

  CloseHandle(hCom);                            // Close COM ressource
  return 0;
}
//------------------------------------------------------------------------------


//cdc added
BOOL PacketMenu(BYTE *Array,DWORD *Arraylength)
{
	char chdecide;
	int input;
	unsigned int i=0;
	BOOL retval=TRUE;
	BOOL menuabort=TRUE;
	memset(Array, 0, MYBYTESIZE);
	do{
	printf("\n\t-- Packet Menu -- configure the data -- then quit\n");
	//printf("[a]: Byte Read [b]: Byte Write [c]: Block Read [d]: Block Write [q]:quit\n> ");
	printf("[a]: Slave Address\t\t\t[q]:quit\n[b]: Byte Read [c]: Byte Write [d]: Block Read [e]: Block Write\n> ");
	printf("[f]: MSPStatus-Submenu\n");
	scanf("%c",&chdecide);
	fflush(stdin);
	NL;
	Array[2] = 0xD8;
	switch(chdecide)
	{
	case 'a': //slave address
		printf("Slave Address: ");
		printbinary(Array[2],7);
		printf("\nif you already set the Byte Read/Write etc. , you must do it again now");
		printf("\n[a]: CDCE949 [b]:manual [q]: quit > ");
		scanf("%c",&chdecide);
		fflush(stdin);
		switch(chdecide)
		{
		case 'a':
		Array[2] = 0xD8;
		break;
		case 'b':
		printf("Enter Slave Address (7bit,MSB=0) 0x.. : ");
		scanf("%x",&input);
		fflush(stdin);
		Array[2] = input<<1;
		printf("new Slave Address: ");
		printbinary(Array[2],7);
		break;
		case 'q':
			chdecide = 'r';
			break;
		default:
			chdecide= 'r';
		}
		fflush(stdin);
		break;
	case 'b': //byte read
		printf("BYTE READ\tSlave Address: ");
		printbinary(Array[2],7);
		NL;
		Array[0] = PACKET_TYPE_I2C;
		Array[1] = 0x35;
		Array[2] |= 0x01;
		printf("\tEnter address Offset 0x.. : ");
		scanf("%x",&input);
		fflush(stdin);
		input |= 0x80;
		Array[3] = input;
		*Arraylength = 4;
		menuabort=FALSE;
		break;
	case 'c': //byte write
		printf("BYTE WRITE\tSlave Address: ");
		printbinary(Array[2],7);
		NL;
		Array[0] = PACKET_TYPE_I2C;
		Array[1] = 0x2D;
		Array[2] &= ~0x01;
		printf("\tEnter address Offset 0x.. : ");
		scanf("%x",&input);
		fflush(stdin);
		input |= 0x80;
		Array[3] = input;
		printf("\tEnter data byte 0x.. : ");
		scanf("%x",&input);
		fflush(stdin);
		Array[4] = input;
		*Arraylength = 5;
		menuabort=FALSE;
		break;
	case 'd': //block read
		printf("Block Read byte count in device checked? Do you want to set it? [y/n] > ");
		scanf("%c", &chdecide);
		fflush(stdin);
		if(chdecide=='y')
		{
			chdecide='r';
			printf("do a byte write to 0x06 for CDCE949");
		}
		else
		{
			printf("Block Read\tSlave Address: ");
			printbinary(Array[2],7);
			NL;
			Array[0] = PACKET_TYPE_I2C;
			Array[1] = 0x33;
			Array[2] |= 0x01;
			printf("\tEnter address Offset 0x.. : ");
			scanf("%x",&input);
			fflush(stdin);
			Array[3] = input;
			*Arraylength = 4;
		}
		menuabort=FALSE;
		break;
	case 'e': //block write
		// offset 0x20, 8bytes, default data 949: 0x00,0x00,0x00,0x00,0xED,0x02,0x01,0x01;
		printf("Block WRITE\tSlave Address: ");
		printbinary(Array[2],7);
		NL;
		Array[0] = PACKET_TYPE_I2C;
		Array[1] = 0x2B;
		Array[2] &= ~0x01;
		printf("Enter address Offset 0x.. :");
		scanf("%x",&input);
		fflush(stdin);
		Array[3] = input;
		printf("Enter number of bytes (decimal) :");
		scanf("%d",&input);
		fflush(stdin);
		Array[4] = input;
		for(i=0;i<Array[4];i++)
		{
		printf("Enter %d. data byte 0x.. :",i+1);
		scanf("%x",&input);
		fflush(stdin);
		Array[5+i] = input;
		}
		printf("all data in buffer");
		*Arraylength = 5+Array[4];
		menuabort=FALSE;
		break;
	case 'f':
		printf("MSPStatus Submenu\n [a]: Name [b]: Firmware Version [c]: Hardware Version [d]: supp. Packets > ");
		scanf("%c",&chdecide);
		fflush(stdin);
		switch(chdecide)
		{
		case 'a':
			Array[1] = MSPSTAT_NAME;
			break;
		case 'b':
			Array[1] = MSPSTAT_FIRMWARE;
			break;
		case 'c':
			Array[1] = MSPSTAT_HARDWARE;
			break;
		case 'd':
			Array[1] = MSPSTAT_PROTOCOL;
			break;
		default:
			Array[1] = MSPSTAT_NAME;
		}
		Array[0] = PACKET_TYPE_MSPSTAT;		
		*Arraylength = 2;
		menuabort=FALSE;
		chdecide='r';
		break;
	case 'q':
		if(!menuabort)
		{
			printf("\nBuffer to be sent is: ");
			printf("0x%02x ",(unsigned int ) *Arraylength);
			for(i=0;i<*Arraylength;i++)
			{
				printf("0x%02x ",Array[i]);
			}
			printf("\n");
		}
		else
		{
			printf("\tExit I2C\n");
		}
		retval = TRUE;
		break;
	default:
		chdecide= 'r';
	}
	}while((chdecide!='q'));
	fflush(stdin);
	return retval;
}


void printbyte(BYTE *bytes, ULONG bytenum)
{
	ULONG i;
	for(i=0;i<bytenum;i++)
	{
		printf("0x%02x ",bytes[i]);
	}
}

void printbinary(BYTE myByte,int iBits)
{
	//iBits max number of bits, starting msb
	int i;
	int j=7;
	printf("0b");
	for(i=1;i<=iBits;i++)
	{
		if(Splitbyte(myByte, j))
			printf("1");
		else
			printf("0");
		j=j-1;
	}
}

BOOL Splitbyte(BYTE mybyte, int bit) 
{
	BOOL retval;
	switch(bit)
	{
	case 0:
		retval = mybyte & BIT0;
		break;
	case 1:
		retval = (mybyte & BIT1) >> 1;
		break;
	case 2:
		retval = (mybyte & BIT2) >> 2;
		break;
	case 3:
		retval = (mybyte & BIT3) >> 3;
		break;
	case 4:
		retval = (mybyte & BIT4) >> 4;
		break;
	case 5:
		retval = (mybyte & BIT5) >> 5;
		break;
	case 6:
		retval = (mybyte & BIT6) >> 6;
		break;
	case 7:
		retval = (mybyte & BIT7) >> 7;
		break;
	default:
		retval = 0; //SPLITERROR;		
	}
return retval;

}

void wait(int sec)
{
  clock_t stop;
  stop = clock() + sec * CLOCKS_PER_SEC;
  while (clock() < stop) {}
}
